/**** Prepare Required objects ****/

-- Login Procedure
CREATE OR REPLACE PROCEDURE scott.p_login(in_user_tx varchar2) IS
BEGIN
    DBMS_SESSION.SET_IDENTIFIER(in_user_tx);
END;
/
-- Maintenance procedure
CREATE OR REPLACE PROCEDURE scott.p_updateSal(in_empno_nr number, in_sal_nr number) IS
BEGIN
    dbms_application_info.set_module('SALARY_MAINT', 'UPDATE');
    UPDATE scott.emp
    SET sal = in_sal_nr
    WHERE empno = in_empno_nr;
    dbms_application_info.set_module(null,null);
END;
/

/**** Initiate Logging ****/
BEGIN
    dbms_monitor.client_id_trace_enable
        (client_id=>'Misha');   -- monitor all activity for a user  
    dbms_monitor.serv_mod_act_trace_enable(
        service_name => 'SYS$USERS',
        module_name => 'SALARY_MAINT'); -- monitor specific module
END;
/

/**** Run various activities simularing two stateless users ****/

----------------- User #1 -------------------
-- activity A
CONNECT SCOTT/TIGER@ora11g
EXEC p_login('Misha')
SELECT sal FROM scott.emp WHERE empno=7369;

-- activity B
CONNECT SCOTT/TIGER@ora11g
EXEC p_login('Misha')
EXEC p_updateSal(7369,1000)

----------------- User #2 -------------------
-- activity C
CONNECT SCOTT/TIGER@ora11g
EXEC p_login('John')
SELECT sal FROM scott.emp WHERE empno=7499;

-- activity D
CONNECT SCOTT/TIGER@ora11g
EXEC p_login('John')
EXEC p_updateSal(7499,2000)

/**** Disable Logging ****/
CONNECT SCOTT/TIGER@ora11g
BEGIN
    dbms_monitor.client_id_trace_disable
        (client_id=>'Misha'); 
    dbms_monitor.serv_mod_act_trace_disable(
        service_name => 'SYS$USERS',
        module_name => 'SALARY_MAINT'); -- monitor specific module
END;
/

/*
-- Check your trace folder, review newly created files, aggregate as needed:
trcsess output=c:\temp\misha_client.trc C:\app\diag\rdbms\ora11g\ora11g\trace\*.trc clientid=Misha
trcsess output=c:\temp\salary_module.trc C:\app\diag\rdbms\ora11g\ora11g\trace\*.trc module=SALARY_MAINT
*/