/* User-Driven Logging Environment */
CREATE SEQUENCE log_seq
/
CREATE TABLE t_log (
id_nr NUMBER,
timestamp_dt TIMESTAMP,
log_tx VARCHAR2(4000),
log_cl CLOB,
current_user VARCHAR2(32) DEFAULT sys_context('USERENV','CURRENT_USER'),
ip_address VARCHAR2(256) DEFAULT sys_context('USERENV','IP_ADDRESS')
)
/
CREATE OR REPLACE PACKAGE log_pkg IS
    PROCEDURE p_log (i_tx VARCHAR2);
    PROCEDURE p_log (i_cl CLOB);
END;
/
CREATE OR REPLACE PACKAGE BODY log_pkg IS
    PROCEDURE p_log (i_tx VARCHAR2) IS
        pragma autonomous_transaction;
    BEGIN
        INSERT INTO t_log (id_nr, timestamp_dt, log_tx, log_cl)
        VALUES (log_seq.nextval, systimestamp,
                CASE WHEN LENGTH(i_tx)<=4000 THEN i_tx ELSE NULL END,
                CASE WHEN LENGTH(i_tx)>4000 THEN i_tx ELSE NULL END);
        COMMIT;
    END;
    
    PROCEDURE p_log (i_cl CLOB) IS
        pragma autonomous_transaction;
    BEGIN
        INSERT INTO t_log (id_nr, timestamp_dt, log_cl)
        VALUES (log_seq.nextval, systimestamp, i_cl);
        COMMIT;
    END;
END;
/
/* Logging-Enable module*/
CREATE OR REPLACE FUNCTION f_getDeptInfo_tx (i_deptno number) 
RETURN VARCHAR2 IS
    v_out_tx VARCHAR2(256);
BEGIN
    $IF $$MishaDebug $THEN
        log_pkg.p_log ('F_GetDeptInfo_Param:'||i_deptno);
    $END

    SELECT deptno
    INTO v_out_tx
    FROM scott.dept; -- missing WHERE clause
    
    $IF $$MishaDebug $THEN
        log_pkg.p_log ('After query 1');
    $END
    
    SELECT v_out_tx||':'||count(*)
    INTO v_out_tx
    FROM scott.emp
    WHERE deptno = i_deptno;
    
    $IF $$MishaDebug $then
        log_pkg.p_log ('Function end');
    $END
    
    RETURN v_out_tx;
EXCEPTION
    WHEN OTHERS THEN
    log_pkg.p_log(dbms_utility.format_error_stack);
    log_pkg.p_log(dbms_utility.format_error_backtrace);
    RAISE;
END;
/

/* Test Logging */
SELECT f_getDeptInfo_tx(10) FROM dual;

column log_tx format A50
SELECT to_char(timestamp_dt,'HH24:MI:SS') time_dt, log_tx 
FROM t_log
ORDER BY timestamp_dt;

/* Get Extra Logging via Conditonal Compilation */
ALTER FUNCTION f_getDeptInfo_tx COMPILE PLSQL_CCFLAGS='MishaDebug:TRUE';

SELECT f_getDeptInfo_tx(10) FROM dual;

SELECT to_char(timestamp_dt,'HH24:MI:SS') time_dt, log_tx 
FROM t_log
ORDER BY timestamp_dt;
