-- Parameter object type
CREATE OR REPLACE TYPE listAggParam_ot IS OBJECT
	(value_tx VARCHAR2(4000),
	separator_tx VARCHAR2(10))
/

-- Implementation type
CREATE OR REPLACE TYPE ListAggCLImpl AS OBJECT (
    v_out_cl CLOB,
    v_defaultSeparator_tx VARCHAR2(10),

    STATIC FUNCTION ODCIAggregateInitialize
        (sctx IN OUT ListAggCLImpl)
    RETURN NUMBER,
    
    MEMBER FUNCTION ODCIAggregateIterate
        (self IN OUT ListAggCLImpl,
        value_ot IN listAggParam_ot) 
    RETURN NUMBER,
    
    MEMBER FUNCTION ODCIAggregateTerminate
        (self IN ListAggCLImpl,
        returnValue OUT CLOB, 
        flags IN NUMBER) 
    RETURN NUMBER,
    
    MEMBER FUNCTION ODCIAggregateMerge
        (self IN OUT ListAggCLImpl,
        ctx2 IN ListAggCLImpl) 
    RETURN NUMBER
);
/
CREATE OR REPLACE TYPE BODY ListAggCLImpl is
    STATIC FUNCTION ODCIAggregateInitialize
        (sctx IN OUT ListAggCLImpl)
    RETURN NUMBER IS
    BEGIN
        sctx := ListAggCLImpl(null,','); -- default constructor
        RETURN ODCIConst.Success;
    END;
    
    MEMBER FUNCTION ODCIAggregateIterate
        (self IN OUT ListAggCLImpl, 
        value_ot IN listAggParam_ot)
    RETURN NUMBER IS
    BEGIN
        IF self.v_out_cl IS NULL THEN
            self.v_defaultSeparator_tx:=value_ot.separator_tx;
            dbms_lob.createtemporary(self.v_out_cl,true,dbms_lob.call);
            dbms_lob.writeappend
                (self.v_out_cl,length(value_ot.value_tx),value_ot.value_tx);
        ELSE
            dbms_lob.writeappend
                (self.v_out_cl,
                 length(value_ot.separator_tx||value_ot.value_tx),
                 value_ot.separator_tx||value_ot.value_tx);
        END IF;
        RETURN ODCIConst.Success;
    END;
    
    MEMBER FUNCTION ODCIAggregateTerminate
        (self IN ListAggCLImpl, 
         returnValue OUT CLOB, flags IN NUMBER)
    RETURN NUMBER IS
    BEGIN
        returnValue := self.v_out_cl;
    RETURN ODCIConst.Success;
    END;
    
    MEMBER FUNCTION ODCIAggregateMerge
        (self IN OUT ListAggCLImpl, 
         ctx2 IN ListAggCLImpl)
    RETURN NUMBER IS
    BEGIN
        IF ctx2.v_out_cl IS NOT NULL THEN
            IF self.v_out_cl IS NULL THEN
                self.v_out_cl:=ctx2.v_out_cl;
            ELSE
                dbms_lob.writeappend(
                        self.v_out_cl,
                        length(self.v_defaultSeparator_tx),
                        self.v_defaultSeparator_tx);
                dbms_lob.append(self.v_out_cl,ctx2.v_out_cl);
            END IF;
        END IF;
        RETURN ODCIConst.Success;
    END;
END;
/

-- Aggregate Function
CREATE OR REPLACE FUNCTION ListAggCL (value_ot listAggParam_ot)
RETURN CLOB
PARALLEL_ENABLE
AGGREGATE USING ListAggCLImpl;
/

-- Examples of usage
column list_cl format A50
SELECT deptno, ListAggCL(listAggParam_ot(ename,',')) list_cl
FROM emp
GROUP BY deptno;

SELECT empno, 
        ename, 
        ListAggCL(listAggParam_ot(ename,','))
            OVER(PARTITION BY deptno ORDER BY ename
            ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) list_cl
FROM emp
WHERE job = 'CLERK';

