/* Sample set of functions */ 
CREATE OR REPLACE PACKAGE perf_pkg IS
    FUNCTION f_heavy_tx (i_deptno NUMBER) RETURN VARCHAR2;
    FUNCTION f_light_tx (i_empno NUMBER) RETURN VARCHAR2;
    FUNCTION f_medium_tx (i_name VARCHAR2) RETURN VARCHAR2;
end;
/
CREATE OR REPLACE PACKAGE BODY perf_pkg IS
    FUNCTION f_heavy_tx (i_deptno number) 
    RETURN VARCHAR2 IS
    BEGIN 
        RETURN 'Heavy:'||i_deptno; 
    END;
    
    FUNCTION f_light_tx (i_empno number) RETURN VARCHAR2 IS
    BEGIN 
        RETURN 'Light:'||i_empno; 
    END;
    
    FUNCTION f_medium_tx (i_name varchar2) RETURN VARCHAR2 IS
    BEGIN 
        RETURN initcap(i_name); 
    END;
END;
/

/* Object type needed to associated statistics*/
CREATE OR REPLACE TYPE function_stat_oty AS OBJECT (
    dummy_attribute NUMBER,
    STATIC FUNCTION ODCIGetInterfaces
        (p_interfaces OUT SYS.ODCIObjectList)
    RETURN NUMBER,
    
    STATIC FUNCTION ODCIStatsFunctionCost (
        p_func_info IN SYS.ODCIFuncInfo,
        p_cost OUT SYS.ODCICost,
        p_args IN SYS.ODCIArgDescList,
        i_single_nr IN NUMBER,
        p_env IN SYS.ODCIEnv
        ) 
    RETURN NUMBER,
    
    STATIC FUNCTION ODCIStatsFunctionCost (
        p_func_info IN SYS.ODCIFuncInfo,
        p_cost OUT SYS.ODCICost,
        p_args IN SYS.ODCIArgDescList,
        i_single_tx IN VARCHAR2,
        p_env IN SYS.ODCIEnv
        ) RETURN NUMBER
)
/
CREATE OR REPLACE TYPE BODY function_stat_oty AS
    STATIC FUNCTION ODCIGetInterfaces 
        (p_interfaces OUT SYS.ODCIObjectList)
    RETURN NUMBER IS
    BEGIN
        p_interfaces := SYS.ODCIObjectList(SYS.ODCIObject ('SYS', 'ODCISTATS2'));
        RETURN ODCIConst.success;
    END ODCIGetInterfaces;
    
    STATIC FUNCTION ODCIStatsFunctionCost 
        (p_func_info IN SYS.ODCIFuncInfo,
        p_cost OUT SYS.ODCICost,
        p_args IN SYS.ODCIArgDescList,
        i_single_nr IN NUMBER,
        p_env IN SYS.ODCIEnv
        ) 
    RETURN NUMBER IS
    BEGIN
        IF UPPER(p_func_info.MethodName) LIKE '%HEAVY%' THEN
            p_cost := SYS.ODCICost
                            (CPUcost=>NULL, 
                             IOcost=>1000, 
                             NetworkCost=>NULL, 
                             IndexCostInfo=>NULL);
        END IF;
        RETURN ODCIConst.success;
    END;
    
    STATIC FUNCTION ODCIStatsFunctionCost 
        (p_func_info IN SYS.ODCIFuncInfo,
        p_cost OUT SYS.ODCICost,
        p_args IN SYS.ODCIArgDescList,
        i_single_tx in varchar2,
        p_env IN SYS.ODCIEnv
        ) 
    RETURN NUMBER IS
    BEGIN
        IF UPPER(p_func_info.MethodName) LIKE '%MEDIUM%' then
            p_cost := SYS.ODCICost(NULL, 10, NULL, NULL);
        END IF;
        RETURN ODCIConst.success;
    END;
END;    
/

/* Validate Impact*/
ASSOCIATE STATISTICS WITH PACKAGES perf_pkg USING function_stat_oty
/
SET AUTOTRACE ON EXPLAIN

SELECT count(*) FROM EMP
WHERE perf_pkg.f_heavy_tx(empno)='A'
OR perf_pkg.f_light_tx(deptno)='B'
OR perf_pkg.f_medium_tx(job)='C'
/
