-- create required objects
CREATE TABLE inlist_tab AS
SELECT object_id, created, object_type
FROM all_objects
WHERE object_id IS NOT NULL
/
ALTER TABLE inlist_tab
ADD CONSTRAINT inlist_tab_pk PRIMARY KEY (object_id) USING INDEX
/
BEGIN
dbms_stats.gather_table_stats(user,'INLIST_TAB');
END;
/
-- create object collection
CREATE TYPE id_tt IS TABLE OF NUMBER
/

-- check original execution plan
SELECT /*+ gather_plan_statistics CH05_CARD_BEFORE */ max(created)
WHERE object_id IN (
            SELECT t.column_value
            FROM table(
                    mycard(id_tt(100,101))
                    ) t
            )
/
select *
from table (dbms_xplan.display_cursor
                ((select sql_id
                  from v$sqlarea
                  where sql_text like '%CH05_CARD_BEFORE%'
                  and sql_text not like '%v$sqlarea%'),
                 null,
                 'ALLSTATS')
           )
/

-- Check hint impact
SELECT /*+ gather_plan_statistics CH05_CARD_HINT */ MAX(created)
FROM inlist_tab
WHERE object_id IN (
            SELECT t.column_value
            FROM table(
                    mycard(id_tt(100,101))
                    ) t
            )
/
select *
from table (dbms_xplan.display_cursor
                ((select sql_id
                  from v$sqlarea
                  where sql_text like '%CH05_CARD_HINT%'
                  and sql_text not like '%v$sqlarea%'),
                 null,
                 'ALLSTATS')
           )
/

-- Create Statistics Gatherer
CREATE OR REPLACE FUNCTION mycard(i_tt id_tt)
RETURN id_tt IS
BEGIN
    RETURN i_tt;
END;
/
CREATE TYPE MyCard_OT AS OBJECT (
    dummy_attribute NUMBER,
    STATIC FUNCTION ODCIGetInterfaces 
        (p_interfaces OUT SYS.ODCIObjectList)
    RETURN NUMBER,
    
    STATIC FUNCTION ODCIStatsTableFunction (
        p_function IN SYS.ODCIFuncInfo,
        p_stats OUT SYS.ODCITabFuncStats,
        p_args IN SYS.ODCIArgDescList,
        i_tt IN id_tt)
    RETURN NUMBER
)
/
CREATE TYPE BODY MyCard_OT AS
    STATIC FUNCTION ODCIGetInterfaces 
        (p_interfaces OUT SYS.ODCIObjectList)
    RETURN NUMBER IS
    BEGIN
        p_interfaces := SYS.ODCIObjectList(SYS.ODCIObject('SYS', 'ODCISTATS2'));
        RETURN ODCIConst.success;
    END ODCIGetInterfaces;
    
    STATIC FUNCTION ODCIStatsTableFunction 
        (p_function IN SYS.ODCIFuncInfo,
        p_stats OUT SYS.ODCITabFuncStats,
        p_args IN SYS.ODCIArgDescList,
        i_tt IN id_tt) 
    RETURN NUMBER IS
    BEGIN
        p_stats := SYS.ODCITabFuncStats(i_tt.COUNT); -- set cardinality
        RETURN ODCIConst.success;
    END ODCIStatsTableFunction;
END;
/
ASSOCIATE STATISTICS WITH FUNCTIONS MyCard USING MyCard_ot
/
-- Valiate impact of Statistics
SELECT /*+ gather_plan_statistics CH05_CARD_AFTER */ MAX(created)
FROM inlist_tab
WHERE object_id IN (
            SELECT t.column_value
            FROM table(
                    mycard(id_tt(100,101))
                    ) t
            )
/
select *
from table (dbms_xplan.display_cursor
                ((select sql_id
                  from v$sqlarea
                  where sql_text like '%CH05_CARD_AFTER%'
                  and sql_text not like '%v$sqlarea%'),
                 null,
                 'ALLSTATS')
           )
/
