-- Sample Function
CREATE OR REPLACE FUNCTION f_isSenior_yn (i_job_id VARCHAR2) 
RETURN VARCHAR2
IS
BEGIN
    IF i_job_id IN ('AD_PRES','AD_VP') THEN
        RETURN 'Y';
    ELSE
        RETURN 'N';
    END IF;
END;
/
-- Object type to gather statistics
CREATE OR REPLACE TYPE MySelect_OT AS OBJECT (
    dummy_attribute NUMBER,
    STATIC FUNCTION ODCIGetInterfaces 
        (p_interfaces OUT SYS.ODCIObjectList)
    RETURN NUMBER,
    
    STATIC FUNCTION ODCIStatsSelectivity (
        p_pred_info IN SYS.ODCIPredInfo,
        p_selectivity OUT NUMBER,
        p_args IN SYS.ODCIArgDescList,
        p_start IN VARCHAR2,
        p_stop IN VARCHAR2,
        i_job IN VARCHAR2,
        p_env IN SYS.ODCIEnv)
    RETURN NUMBER
)
/
CREATE OR REPLACE TYPE BODY MySelect_OT AS
    STATIC FUNCTION ODCIGetInterfaces 
        (p_interfaces OUT SYS.ODCIObjectList)
    RETURN NUMBER IS
    BEGIN
        p_interfaces := SYS.ODCIObjectList(SYS.ODCIObject('SYS', 'ODCISTATS2'));
        RETURN ODCIConst.success;   
    END ODCIGetInterfaces;
    
    STATIC FUNCTION ODCIStatsSelectivity (
        p_pred_info IN SYS.ODCIPredInfo,
        p_selectivity OUT NUMBER,
        p_args IN SYS.ODCIArgDescList,
        p_start IN VARCHAR2,
        p_stop IN VARCHAR2,
        i_job IN VARCHAR2,
        p_env IN SYS.ODCIEnv
        )
    RETURN NUMBER IS
    BEGIN
        if p_start='Y' then
            p_selectivity:=3;
        else
            p_selectivity:=97;
        end if;
        RETURN ODCIConst.success;
    END ODCIStatsSelectivity;
END;
/
-- Original execution plan check
SELECT /*+ gather plan statistics CH05_SEL_BEFORE */ employees.emploee_id,
        departments.department_name
FROM hr.employees,
     hr.departments
WHERE employees.department_id = departments.department_id
AND f_isSenior_yn(employees.job_id)='N'
/
select *
from table (dbms_xplan.display_cursor
                ((select sql_id
                  from v$sqlarea
                  where sql_text like '%CH05_SEL_BEFORE%'
                  and sql_text not like '%v$sqlarea%'),
                 null,
                 'ALLSTATS')
           )
/

-- Associate statistics and check the impact
ASSOCIATE STATISTICS WITH FUNCTIONS f_isSenior_yn USING MySelect_OT
/
SELECT /*+ gather plan statistics CH05_SEL_AFTER */ employees.emploee_id,
        departments.department_name
FROM hr.employees,
     hr.departments
WHERE employees.department_id = departments.department_id
AND f_isSenior_yn(employees.job_id)='N'
/
select *
from table (dbms_xplan.display_cursor
                ((select sql_id
                  from v$sqlarea
                  where sql_text like '%CH05_SEL_AFTER%'
                  and sql_text not like '%v$sqlarea%'),
                 null,
                 'ALLSTATS')
           )
/


-- Object type to gather statistics
ASSOCIATE STATISTICS WITH FUNCTIONS f_isSenior_yn USING MySelect_OT
/
-- Validate the impact