-- create local table if you cannot access DB-link
-- in that case in addition to PGA you would need to monitor

create table test_tab_remote
as
select *
from test_tab
/
-- Query to review statistics
SELECT a.name, b.value
FROM v$statname a, v$mystat b
WHERE a.statistic# = b.statistic#
and value!=0

/* Various test cases*/


-- #1 Single-column bulk collect
connect scott/TIGER@ora11g
DECLARE
    TYPE number_tt IS TABLE OF NUMBER;
    v_tt number_tt;
    v_nr NUMBER;
BEGIN
    SELECT object_id
    BULK COLLECT INTO v_tt
    FROM test_tab_remote;
    
    FOR i IN v_tt.first..v_tt.last LOOP
        v_nr:=v_tt(i);
    END LOOP;
END;
/

-- #2 Single-column loop
connect scott/TIGER@ora11g
DECLARE
    v_nr NUMBER;
BEGIN
    FOR c IN (SELECT object_id FROM test_tab_remote) LOOP
        v_nr :=c.object_id;
    END LOOP;
END;
/
-- #3 Multi-column bulk collect
DECLARE
    TYPE number_tt IS TABLE OF test_tab_remote%rowtype;
    v_tt number_tt;
    v_nr NUMBER;
BEGIN
    SELECT *
    BULK COLLECT INTO v_tt
    FROM test_tab_remote;
    
    FOR i IN v_tt.first..v_tt.last LOOP
        v_nr:=v_tt(i).object_id;
    END LOOP;
END;
/

--#4 Multi-column bulk collect
connect scott/TIGER@ora11g
DECLARE
    v_nr NUMBER;
BEGIN
    FOR c IN (SELECT * FROM test_tab_remote) LOOP
        v_nr :=c.object_id;
    END LOOP;
END;
/

-- #5limit clause
connect scott/TIGER@ora11g
DECLARE
    TYPE number_tt IS TABLE OF test_tab_remote%ROWTYPE;
    v_tt number_tt;
    v_nr NUMBER;
    v_cur SYS_REFCURSOR;
    v_limit_nr CONSTANT BINARY_INTEGER:=5000;
BEGIN
    OPEN v_cur FOR SELECT * FROM test_tab_remote;
    LOOP
        FETCH v_cur BULK COLLECT INTO v_tt LIMIT v_limit_nr;
        EXIT WHEN v_tt.count()=0;
        FOR i IN v_tt.first..v_tt.last LOOP
            v_nr:=v_tt(i).object_id;
        END LOOP;
        EXIT WHEN v_tt.count<v_limit_nr;
    END LOOP;
    CLOSE v_cur;
END;
/