-- Create required objects
CREATE TABLE test_tab2 AS SELECT * FROM test_tab;
/
-- object type and collection
CREATE TYPE test_tab_ot AS OBJECT (owner_tx VARCHAR2(30),
        name_tx VARCHAR2(30),
        object_id NUMBER,
        type_tx VARCHAR2(30));
/        
CREATE TYPE test_tab_nt IS TABLE OF test_tab_ot;
/
-- search against TEST_TAB
CREATE OR REPLACE FUNCTION f_searchTestTab_tt (i_type_tx VARCHAR2) RETURN test_tab_nt IS
v_out_tt test_tab_nt;
BEGIN
    SELECT test_tab_ot(owner, object_name, object_id, object_type)
    BULK COLLECT INTO v_out_tt
    FROM test_tab WHERE object_type = i_type_tx;
    RETURN v_out_tt;
END;
/
-- search against TEST_TAB2
CREATE OR REPLACE FUNCTION f_searchTestTab2_tt (i_type_tx VARCHAR2) RETURN test_tab_nt IS
    v_out_tt test_tab_nt;
BEGIN
    SELECT test_tab_ot(owner, object_name, object_id, object_type)
    BULK COLLECT INTO v_out_tt
    FROM test_tab2 WHERE object_type = i_type_tx;
    RETURN v_out_tt;
END;
/
CREATE OR REPLACE FUNCTION f_searchTestUnion_tt (i_type1_tx VARCHAR2, i_type2_tx VARCHAR2)
RETURN test_tab_nt IS
    v_type1_tt test_tab_nt;
    v_type2_tt test_tab_nt;
    v_out_tt test_tab_nt;
BEGIN
    SELECT test_tab_ot(owner, object_name, object_id, object_type)
    BULK COLLECT INTO v_type1_tt
    FROM test_tab
    WHERE object_type = i_type1_tx;

    SELECT test_tab_ot(owner, object_name, object_id, object_type)
    BULK COLLECT INTO v_type2_tt
    FROM test_tab2
    WHERE object_type = i_type2_tx;
    v_out_tt:= v_type1_tt MULTISET UNION ALL v_type2_tt;
    RETURN v_out_tt;
END;
/

-- Option #1 SQL union
select max(object_id), min(object_id), count(*)
from (select * from test_tab where object_type = 'SYNONYM'
      union all
      select * from test_tab2 where object_type = 'JAVA CLASS')
/
SELECT a.name, b.value
FROM v$statname a, v$mystat b
WHERE a.statistic# = b.statistic#
AND name = 'session pga memory max'
/

-- Option #2 SQL union of collections
select max(object_id), min(object_id), count(*)
from (select * from table(f_searchTestTab_tt('SYNONYM'))
      union all   
      select * from table(f_searchTestTab2_tt('JAVA CLASS')))
/
SELECT a.name, b.value
FROM v$statname a, v$mystat b
WHERE a.statistic# = b.statistic#
AND name = 'session pga memory max'
/

-- Option #3 Multiset union
select max(object_id), min(object_id), count(*)
from table(
        f_searchTestUnion_tt('SYNONYM','JAVA CLASS')
        )
/
SELECT a.name, b.value
FROM v$statname a, v$mystat b
WHERE a.statistic# = b.statistic#
AND name = 'session pga memory max'
/