-- Create tables
create table test_tab_main
as
select *
from test_tab
where rownum < 1;
/
create table test_tab_other
as
select *
from test_tab
where rownum < 1;
/
-- Use local tables instead of remote (to simplify the example)
CREATE OR REPLACE PROCEDURE p_testTab_regular IS
    TYPE table_tt IS TABLE OF test_tab%rowtype;
    v_tt table_tt;
    v_cur SYS_REFCURSOR;
BEGIN
    OPEN v_cur FOR SELECT * FROM test_tab;
    LOOP
        FETCH v_cur BULK COLLECT INTO v_tt LIMIT 5000;
        EXIT WHEN v_tt.count()=0;
        FOR i IN v_tt.first..v_tt.last LOOP
            IF v_tt(i).object_type='TABLE' THEN
                INSERT INTO test_tab_main VALUES v_tt(i);
            ELSE
                INSERT INTO test_tab_other VALUES v_tt(i);
            END IF;
        END LOOP;
        EXIT WHEN v_tt.count<5000;
    END LOOP;
    CLOSE v_cur;
END;
/
CREATE OR REPLACE PROCEDURE p_testTab_ForAll IS
    TYPE table_tt IS TABLE OF test_tab%ROWTYPE;
    TYPE number_tt IS TABLE OF BINARY_INTEGER;
    v_tt table_tt;
    v_cur SYS_REFCURSOR;
    v_main_tt number_tt;
    v_other_tt number_tt;
BEGIN
    OPEN v_cur FOR SELECT * FROM test_tab;
    LOOP
        v_main_tt:=number_tt();
        v_other_tt:=number_tt();
        FETCH v_cur BULK COLLECT INTO v_tt LIMIT 5000;
        EXIT WHEN v_tt.count()=0;
        FOR i IN v_tt.first..v_tt.last LOOP
            IF v_tt(i).object_type='TABLE' THEN
                v_main_tt.extend;
                v_main_tt(v_main_tt.last):=i;
            ELSE
                v_other_tt.extend;
                v_other_tt(v_other_tt.last):=i;
            END IF;
        END LOOP;    

        FORALL i IN VALUES OF v_main_tt
            INSERT INTO test_tab_main VALUES v_tt(i);
    
        FORALL i IN VALUES OF v_other_tt
            INSERT INTO test_tab_other VALUES v_tt(i);
        
        EXIT WHEN v_tt.count<5000;
    END LOOP;
    CLOSE v_cur;
END;
/
-- compare results
exec runstats_pkg.rs_start;
exec p_testtab_forall;
exec runstats_pkg.rs_middle;
exec p_testtab_regular;
exec runstats_pkg.rs_stop;