-- Required objects
CREATE TABLE emp2 AS
SELECT empno*10 empno, deptno FROM emp
/
CREATE OR REPLACE TRIGGER EMP_STATEMENT_BIUD
BEFORE INSERT OR DELETE OR UPDATE ON EMP
BEGIN
    IF INSERTING THEN dbms_output.put_line('***Statement-before-INSERT');
    ELSIF UPDATING THEN dbms_output.put_line('***Statement-before-UPDATE');
    ELSIF DELETING THEN dbms_output.put_line('***Statement-before-DELETE');
    END IF;
END;
/
CREATE OR REPLACE TRIGGER EMP_STATEMENT_AIUD
AFTER INSERT OR DELETE OR UPDATE ON EMP
BEGIN
    IF INSERTING THEN dbms_output.put_line('***Statement-after-INSERT');
    ELSIF UPDATING THEN dbms_output.put_line('***Statement-after-UPDATE');
    ELSIF DELETING THEN dbms_output.put_line('***Statement-after-DELETE');
    END IF;
END;
/
CREATE OR REPLACE TRIGGER EMP_ROW_BIUD
BEFORE INSERT OR DELETE OR UPDATE ON EMP FOR EACH ROW
BEGIN
    IF INSERTING THEN dbms_output.put_line('Row-before-INSERT');
    ELSIF UPDATING THEN dbms_output.put_line('Row-before-UPDATE');
    ELSIF DELETING THEN dbms_output.put_line('Row-before-DELETE');
    END IF;
END;
/
CREATE OR REPLACE TRIGGER EMP_ROW_AIUD
AFTER INSERT OR DELETE OR UPDATE ON EMP FOR EACH ROW
BEGIN
    IF INSERTING THEN dbms_output.put_line('Row-after-INSERT');
    ELSIF UPDATING THEN dbms_output.put_line('Row-after-UPDATE');
    ELSIF DELETING THEN dbms_output.put_line('Row-after-DELETE');
    END IF;
END;
/
CREATE OR REPLACE PROCEDURE p_check_forall IS
    TYPE tt IS TABLE OF NUMBER;
    v_dml_tt tt:=tt(10,20);
    TYPE table_tt IS TABLE OF emp%ROWTYPE ;
    v_insert_tt table_tt:=table_tt();
BEGIN
    dbms_output.put_line('--------Update----------');
    FORALL i IN INDICES OF v_dml_tt
        UPDATE emp SET ename = UPPER(ename) WHERE deptno = v_dml_tt(i);
        
    dbms_output.put_line('--------InsertRow----------');
    v_insert_tt.extend;
    v_insert_tt(v_insert_tt.last).empno:=100;
    v_insert_tt(v_insert_tt.last).deptno:=10;
    v_insert_tt.extend;
    v_insert_tt(v_insert_tt.last).empno:=200;
    v_insert_tt(v_insert_tt.last).deptno:=20;
    
    FORALL I IN v_insert_tt.first..v_insert_tt.last
        INSERT INTO emp VALUES v_insert_tt(i);
        
    dbms_output.put_line('--------InsertAsSelect----------');
    FORALL i IN INDICES OF v_dml_tt
        INSERT INTO emp (empno,deptno)
        SELECT empno,deptno 
        FROM emp2 
        WHERE deptno = v_dml_tt(i);
    
    dbms_output.put_line('--------Delete-----------');
    FORALL i IN INDICES OF v_dml_tt
        DELETE FROM emp WHERE deptno = v_dml_tt (i);
END;
/
set serveroutput on;
exec p_check_forall;
