-- Prepare data with known issues
ALTER TABLE test_tab ADD last_ddl_tx VARCHAR2(8)
/
UPDATE test_tab SET last_ddl_tx=to_char(last_ddl_time,'YYYYMMDD')
/
UPDATE test_tab SET last_ddl_tx='DEFAULT' WHERE rownum < 5
/

-- Prepare function-based columns
CREATE FUNCTION f_to_date 
    (i_tx VARCHAR2, 
    i_format_tx VARCHAR2:='YYYYMMDD')
RETURN DATE DETERMINISTIC IS
BEGIN
    RETURN to_date(i_tx,i_format_tx);
EXCEPTION
    WHEN OTHERS THEN
        IF SQLCODE BETWEEN -1899 AND -1800 THEN
            RETURN NULL;
        ELSE
            RAISE;
        END IF;
END;
/
CREATE FUNCTION f_to_date_udf 
    (i_tx VARCHAR2, 
    i_format_tx VARCHAR2:='YYYYMMDD')
RETURN DATE DETERMINISTIC IS
    PRAGMA UDF;
BEGIN
    RETURN to_date(i_tx,i_format_tx);
EXCEPTION
    WHEN OTHERS THEN
        IF sqlcode BETWEEN -1899 AND -1800 THEN
            RETURN NULL;
        ELSE
            RAISE;
        END IF;
END;
/
ALTER TABLE test_tab
    ADD (last_ddl_dt AS (f_to_date(last_ddl_tx)),
         last_ddl_udf_dt AS (f_to_date_udf(last_ddl_tx))
        )
/        

-- Check the impact
exec runstats_pkg.rs_start;
SELECT min(last_ddl_dt) FROM test_tab;
exec runstats_pkg.rs_middle;
SELECT min(last_ddl_udf_dt) FROM test_tab;
exec runstats_pkg.rs_stop;
