-- Parametrized search via global variable
CREATE PACKAGE global_pkg IS
    v_object_id NUMBER;
END;
/
CREATE OR REPLACE FUNCTION f_searchTestTab_tt (i_type_tx varchar2) 
RETURN test_tab_nt is
    v_out_tt test_tab_nt;
BEGIN
    IF global_pkg.v_object_id IS NULL THEN
        SELECT test_tab_ot(owner, object_name, object_id, object_type)
        BULK COLLECT INTO v_out_tt
        FROM test_tab
        WHERE object_type = i_type_tx;
    ELSE
        SELECT test_tab_ot(owner, object_name, object_id, object_type)
        BULK COLLECT INTO v_out_tt
        FROM test_tab
        WHERE object_id = global_pkg.v_object_id;
    END IF;
    dbms_output.put_line('Inside f_searchTestTab_tt:'||v_out_tt.count);
    RETURN v_out_tt;
END;
/
-- Validate the functionality
BEGIN global_pkg.v_object_id:=5; END;
/
UPDATE v_search_table SET name_tx = 'Test' WHERE object_id = 5
/
DELETE FROM v_search_table WHERE object_id = 5
/
--- Performance impact of parametrized filtering
exec runstats_pkg.rs_start;
BEGIN
    global_pkg.v_object_id:=NULL;
    FOR i IN 1..100 LOOP
        UPDATE v_search_table SET name_tx = 'Test'||i WHERE object_id = 5;
    END LOOP;
END;
/
exec runstats_pkg.rs_middle;
BEGIN
    global_pkg.v_object_id:=5;
    FOR i IN 1..100 LOOP
        UPDATE v_search_table SET name_tx = 'Test'||i WHERE object_id = 5;
    END LOOP;
END;
/
exec runstats_pkg.rs_stop;

