-- Create view with two kinds of triggers
CREATE OR REPLACE VIEW v_test_tab AS
SELECT *
FROM test_tab_main
UNION ALL
SELECT *
FROM test_tab_other
/
CREATE OR REPLACE TRIGGER v_test_tab_II
INSTEAD OF INSERT ON v_test_tab
BEGIN
    IF :new.object_type='TABLE' THEN
        INSERT INTO test_tab_main
            (owner,object_name,subobject_name,object_id,
            data_object_id,object_type,created,last_ddl_time,
            timestamp,status,temporary,generated,secondary,
            namespace,edition_name,sharing,editionable,
            oracle_maintained)
        VALUES (:NEW.owner,:NEW.object_name,:NEW.subobject_name,:NEW.object_id,
            :NEW.data_object_id,:NEW.object_type,:NEW.created,:NEW.last_ddl_time,
            :NEW.timestamp,:NEW.status,:NEW.temporary,:NEW.generated,:NEW.secondary,
            :NEW.namespace,:NEW.edition_name,:NEW.sharing,:NEW.editionable,
            :NEW.oracle_maintained);
    ELSE
        INSERT INTO test_tab_other
            (owner,object_name,subobject_name,object_id,
            data_object_id,object_type,created,last_ddl_time,
            timestamp,status,temporary,generated,secondary,
            namespace,edition_name,sharing,editionable,
            oracle_maintained)
        VALUES (:NEW.owner,:NEW.object_name,:NEW.subobject_name,:NEW.object_id,
            :NEW.data_object_id,:NEW.object_type,:NEW.created,:NEW.last_ddl_time,
            :NEW.timestamp,:NEW.status,:NEW.temporary,:NEW.generated,:NEW.secondary,
            :NEW.namespace,:NEW.edition_name,:NEW.sharing,:NEW.editionable,
            :NEW.oracle_maintained);
    END IF;
END;
/
CREATE OR REPLACE TRIGGER v_test_tab_IU
INSTEAD OF UPDATE ON v_test_tab
BEGIN
    IF :new.object_type='TABLE' THEN
        UPDATE test_tab_main
            SET owner=:NEW.owner, object_name=:NEW.object_name,
            subobject_name=:NEW.subobject_name, object_type=:NEW.object_type,
            created=:NEW.created, last_ddl_time=:NEW.last_ddl_time,
            timestamp=:NEW.timestamp, status=:NEW.status,
            temporary=:NEW.temporary, generated=:NEW.generated,
            secondary=:NEW.secondary, namespace=:NEW.namespace,
            edition_name=:NEW.edition_name, sharing=:NEW.sharing,
            editionable=:NEW.editionable, oracle_maintained=:NEW.oracle_maintained
        WHERE object_id=:OLD.object_id;
    ELSE
        UPDATE test_tab_other
            SET owner=:NEW.owner, object_name=:NEW.object_name,
            subobject_name=:NEW.subobject_name, object_type=:NEW.object_type,
            created=:NEW.created, last_ddl_time=:NEW.last_ddl_time,
            timestamp=:NEW.timestamp, status=:NEW.status,
            temporary=:NEW.temporary, generated=:NEW.generated,
            secondary=:NEW.secondary, namespace=:NEW.namespace,
            edition_name=:NEW.edition_name, sharing=:NEW.sharing,
            editionable=:NEW.editionable, oracle_maintained=:NEW.oracle_maintained
        WHERE object_id=:old.object_id;
    END IF;
END;
/
CREATE OR REPLACE TRIGGER v_test_tab_dynamic_IU
INSTEAD OF UPDATE ON v_test_tab
DECLARE
    v_main_rec test_tab_main%ROWTYPE;
    v_mainUpdate_tx VARCHAR2(32767);
    v_other_rec test_tab_other%ROWTYPE;
    v_otherUpdate_tx VARCHAR2(32767);
BEGIN
    IF :new.object_type='TABLE' THEN
       -- CREATED
        IF :old.CREATED IS NULL AND :new.CREATED IS NOT NULL
        OR :old.CREATED IS NOT NULL and :new.CREATED IS NULL
        OR :old.CREATED!=:new.CREATED THEN
            v_main_rec.CREATED:=:new.CREATED;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' CREATED=v_rec.CREATED';
        END IF;                                                                                                              
       -- DATA_OBJECT_ID
        IF :old.DATA_OBJECT_ID IS NULL AND :new.DATA_OBJECT_ID IS NOT NULL
        OR :old.DATA_OBJECT_ID IS NOT NULL and :new.DATA_OBJECT_ID IS NULL
        OR :old.DATA_OBJECT_ID!=:new.DATA_OBJECT_ID THEN
            v_main_rec.DATA_OBJECT_ID:=:new.DATA_OBJECT_ID;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' DATA_OBJECT_ID=v_rec.DATA_OBJECT_ID';
        END IF;                                 
       -- EDITION_NAME
        IF :old.EDITION_NAME IS NULL AND :new.EDITION_NAME IS NOT NULL
        OR :old.EDITION_NAME IS NOT NULL and :new.EDITION_NAME IS NULL
        OR :old.EDITION_NAME!=:new.EDITION_NAME THEN
            v_main_rec.EDITION_NAME:=:new.EDITION_NAME;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' EDITION_NAME=v_rec.EDITION_NAME';
        END IF;                                                       
       -- EDITIONABLE
        IF :old.EDITIONABLE IS NULL AND :new.EDITIONABLE IS NOT NULL
        OR :old.EDITIONABLE IS NOT NULL and :new.EDITIONABLE IS NULL
        OR :old.EDITIONABLE!=:new.EDITIONABLE THEN
            v_main_rec.EDITIONABLE:=:new.EDITIONABLE;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' EDITIONABLE=v_rec.EDITIONABLE';
        END IF;                                                                  
       -- GENERATED
        IF :old.GENERATED IS NULL AND :new.GENERATED IS NOT NULL
        OR :old.GENERATED IS NOT NULL and :new.GENERATED IS NULL
        OR :old.GENERATED!=:new.GENERATED THEN
            v_main_rec.GENERATED:=:new.GENERATED;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' GENERATED=v_rec.GENERATED';
        END IF;                                                                                        
       -- LAST_DDL_DT
        IF :old.LAST_DDL_DT IS NULL AND :new.LAST_DDL_DT IS NOT NULL
        OR :old.LAST_DDL_DT IS NOT NULL and :new.LAST_DDL_DT IS NULL
        OR :old.LAST_DDL_DT!=:new.LAST_DDL_DT THEN
            v_main_rec.LAST_DDL_DT:=:new.LAST_DDL_DT;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' LAST_DDL_DT=v_rec.LAST_DDL_DT';
        END IF;                                                                  
       -- LAST_DDL_TIME
        IF :old.LAST_DDL_TIME IS NULL AND :new.LAST_DDL_TIME IS NOT NULL
        OR :old.LAST_DDL_TIME IS NOT NULL and :new.LAST_DDL_TIME IS NULL
        OR :old.LAST_DDL_TIME!=:new.LAST_DDL_TIME THEN
            v_main_rec.LAST_DDL_TIME:=:new.LAST_DDL_TIME;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' LAST_DDL_TIME=v_rec.LAST_DDL_TIME';
        END IF;                                            
       -- LAST_DDL_TX
        IF :old.LAST_DDL_TX IS NULL AND :new.LAST_DDL_TX IS NOT NULL
        OR :old.LAST_DDL_TX IS NOT NULL and :new.LAST_DDL_TX IS NULL
        OR :old.LAST_DDL_TX!=:new.LAST_DDL_TX THEN
            v_main_rec.LAST_DDL_TX:=:new.LAST_DDL_TX;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' LAST_DDL_TX=v_rec.LAST_DDL_TX';
        END IF;                                                                  
       -- LAST_DDL_UDF_DT
        IF :old.LAST_DDL_UDF_DT IS NULL AND :new.LAST_DDL_UDF_DT IS NOT NULL
        OR :old.LAST_DDL_UDF_DT IS NOT NULL and :new.LAST_DDL_UDF_DT IS NULL
        OR :old.LAST_DDL_UDF_DT!=:new.LAST_DDL_UDF_DT THEN
            v_main_rec.LAST_DDL_UDF_DT:=:new.LAST_DDL_UDF_DT;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' LAST_DDL_UDF_DT=v_rec.LAST_DDL_UDF_DT';
        END IF;                      
       -- NAMESPACE
        IF :old.NAMESPACE IS NULL AND :new.NAMESPACE IS NOT NULL
        OR :old.NAMESPACE IS NOT NULL and :new.NAMESPACE IS NULL
        OR :old.NAMESPACE!=:new.NAMESPACE THEN
            v_main_rec.NAMESPACE:=:new.NAMESPACE;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' NAMESPACE=v_rec.NAMESPACE';
        END IF;                                                                                        
       -- OBJECT_ID
        IF :old.OBJECT_ID IS NULL AND :new.OBJECT_ID IS NOT NULL
        OR :old.OBJECT_ID IS NOT NULL and :new.OBJECT_ID IS NULL
        OR :old.OBJECT_ID!=:new.OBJECT_ID THEN
            v_main_rec.OBJECT_ID:=:new.OBJECT_ID;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' OBJECT_ID=v_rec.OBJECT_ID';
        END IF;                                                                                        
       -- OBJECT_NAME
        IF :old.OBJECT_NAME IS NULL AND :new.OBJECT_NAME IS NOT NULL
        OR :old.OBJECT_NAME IS NOT NULL and :new.OBJECT_NAME IS NULL
        OR :old.OBJECT_NAME!=:new.OBJECT_NAME THEN
            v_main_rec.OBJECT_NAME:=:new.OBJECT_NAME;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' OBJECT_NAME=v_rec.OBJECT_NAME';
        END IF;                                                                  
       -- OBJECT_TYPE
        IF :old.OBJECT_TYPE IS NULL AND :new.OBJECT_TYPE IS NOT NULL
        OR :old.OBJECT_TYPE IS NOT NULL and :new.OBJECT_TYPE IS NULL
        OR :old.OBJECT_TYPE!=:new.OBJECT_TYPE THEN
            v_main_rec.OBJECT_TYPE:=:new.OBJECT_TYPE;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' OBJECT_TYPE=v_rec.OBJECT_TYPE';
        END IF;                                                                  
       -- ORACLE_MAINTAINED
        IF :old.ORACLE_MAINTAINED IS NULL AND :new.ORACLE_MAINTAINED IS NOT NULL
        OR :old.ORACLE_MAINTAINED IS NOT NULL and :new.ORACLE_MAINTAINED IS NULL
        OR :old.ORACLE_MAINTAINED!=:new.ORACLE_MAINTAINED THEN
            v_main_rec.ORACLE_MAINTAINED:=:new.ORACLE_MAINTAINED;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' ORACLE_MAINTAINED=v_rec.ORACLE_MAINTAINED';
        END IF;
       -- OWNER
        IF :old.OWNER IS NULL AND :new.OWNER IS NOT NULL
        OR :old.OWNER IS NOT NULL and :new.OWNER IS NULL
        OR :old.OWNER!=:new.OWNER THEN
            v_main_rec.OWNER:=:new.OWNER;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' OWNER=v_rec.OWNER';
        END IF;                                                                                                                                    
       -- SECONDARY
        IF :old.SECONDARY IS NULL AND :new.SECONDARY IS NOT NULL
        OR :old.SECONDARY IS NOT NULL and :new.SECONDARY IS NULL
        OR :old.SECONDARY!=:new.SECONDARY THEN
            v_main_rec.SECONDARY:=:new.SECONDARY;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' SECONDARY=v_rec.SECONDARY';
        END IF;                                                                                        
       -- SHARING
        IF :old.SHARING IS NULL AND :new.SHARING IS NOT NULL
        OR :old.SHARING IS NOT NULL and :new.SHARING IS NULL
        OR :old.SHARING!=:new.SHARING THEN
            v_main_rec.SHARING:=:new.SHARING;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' SHARING=v_rec.SHARING';
        END IF;                                                                                                              
       -- STATUS
        IF :old.STATUS IS NULL AND :new.STATUS IS NOT NULL
        OR :old.STATUS IS NOT NULL and :new.STATUS IS NULL
        OR :old.STATUS!=:new.STATUS THEN
            v_main_rec.STATUS:=:new.STATUS;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' STATUS=v_rec.STATUS';
        END IF;                                                                                                                         
       -- SUBOBJECT_NAME
        IF :old.SUBOBJECT_NAME IS NULL AND :new.SUBOBJECT_NAME IS NOT NULL
        OR :old.SUBOBJECT_NAME IS NOT NULL and :new.SUBOBJECT_NAME IS NULL
        OR :old.SUBOBJECT_NAME!=:new.SUBOBJECT_NAME THEN
            v_main_rec.SUBOBJECT_NAME:=:new.SUBOBJECT_NAME;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' SUBOBJECT_NAME=v_rec.SUBOBJECT_NAME';
        END IF;                                 
       -- TEMPORARY
        IF :old.TEMPORARY IS NULL AND :new.TEMPORARY IS NOT NULL
        OR :old.TEMPORARY IS NOT NULL and :new.TEMPORARY IS NULL
        OR :old.TEMPORARY!=:new.TEMPORARY THEN
            v_main_rec.TEMPORARY:=:new.TEMPORARY;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' TEMPORARY=v_rec.TEMPORARY';
        END IF;                                                                                        
       -- TIMESTAMP
        IF :old.TIMESTAMP IS NULL AND :new.TIMESTAMP IS NOT NULL
        OR :old.TIMESTAMP IS NOT NULL and :new.TIMESTAMP IS NULL
        OR :old.TIMESTAMP!=:new.TIMESTAMP THEN
            v_main_rec.TIMESTAMP:=:new.TIMESTAMP;
            v_mainUpdate_tx:=v_mainUpdate_tx||
                                CASE WHEN v_mainUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' TIMESTAMP=v_rec.TIMESTAMP';
        END IF;                                                                                        
    
        v_main_rec.object_id:=:old.object_id;
        v_mainUpdate_tx:=
            'DECLARE '||CHR(10)||
            ' v_rec test_tab_main%ROWTYPE:=:1;'||CHR(10)||
            'BEGIN '||CHR(10)||
            ' UPDATE test_tab_main SET '||
            v_mainUpdate_tx||CHR(10)||
            ' WHERE object_id=v_rec.object_id;'||CHR(10)||
            'END;';
        execute immediate v_mainUpdate_tx using v_main_rec;

    else
       -- CREATED
        IF :old.CREATED IS NULL AND :new.CREATED IS NOT NULL
        OR :old.CREATED IS NOT NULL and :new.CREATED IS NULL
        OR :old.CREATED!=:new.CREATED THEN
            v_other_rec.CREATED:=:new.CREATED;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' CREATED=v_rec.CREATED';
        END IF;                                                                                                              
       -- DATA_OBJECT_ID
        IF :old.DATA_OBJECT_ID IS NULL AND :new.DATA_OBJECT_ID IS NOT NULL
        OR :old.DATA_OBJECT_ID IS NOT NULL and :new.DATA_OBJECT_ID IS NULL
        OR :old.DATA_OBJECT_ID!=:new.DATA_OBJECT_ID THEN
            v_other_rec.DATA_OBJECT_ID:=:new.DATA_OBJECT_ID;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' DATA_OBJECT_ID=v_rec.DATA_OBJECT_ID';
        END IF;                                 
       -- EDITION_NAME
        IF :old.EDITION_NAME IS NULL AND :new.EDITION_NAME IS NOT NULL
        OR :old.EDITION_NAME IS NOT NULL and :new.EDITION_NAME IS NULL
        OR :old.EDITION_NAME!=:new.EDITION_NAME THEN
            v_other_rec.EDITION_NAME:=:new.EDITION_NAME;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' EDITION_NAME=v_rec.EDITION_NAME';
        END IF;                                                       
       -- EDITIONABLE
        IF :old.EDITIONABLE IS NULL AND :new.EDITIONABLE IS NOT NULL
        OR :old.EDITIONABLE IS NOT NULL and :new.EDITIONABLE IS NULL
        OR :old.EDITIONABLE!=:new.EDITIONABLE THEN
            v_other_rec.EDITIONABLE:=:new.EDITIONABLE;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' EDITIONABLE=v_rec.EDITIONABLE';
        END IF;                                                                  
       -- GENERATED
        IF :old.GENERATED IS NULL AND :new.GENERATED IS NOT NULL
        OR :old.GENERATED IS NOT NULL and :new.GENERATED IS NULL
        OR :old.GENERATED!=:new.GENERATED THEN
            v_other_rec.GENERATED:=:new.GENERATED;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' GENERATED=v_rec.GENERATED';
        END IF;                                                                                        
       -- LAST_DDL_DT
        IF :old.LAST_DDL_DT IS NULL AND :new.LAST_DDL_DT IS NOT NULL
        OR :old.LAST_DDL_DT IS NOT NULL and :new.LAST_DDL_DT IS NULL
        OR :old.LAST_DDL_DT!=:new.LAST_DDL_DT THEN
            v_other_rec.LAST_DDL_DT:=:new.LAST_DDL_DT;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' LAST_DDL_DT=v_rec.LAST_DDL_DT';
        END IF;                                                                  
       -- LAST_DDL_TIME
        IF :old.LAST_DDL_TIME IS NULL AND :new.LAST_DDL_TIME IS NOT NULL
        OR :old.LAST_DDL_TIME IS NOT NULL and :new.LAST_DDL_TIME IS NULL
        OR :old.LAST_DDL_TIME!=:new.LAST_DDL_TIME THEN
            v_other_rec.LAST_DDL_TIME:=:new.LAST_DDL_TIME;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' LAST_DDL_TIME=v_rec.LAST_DDL_TIME';
        END IF;                                            
       -- LAST_DDL_TX
        IF :old.LAST_DDL_TX IS NULL AND :new.LAST_DDL_TX IS NOT NULL
        OR :old.LAST_DDL_TX IS NOT NULL and :new.LAST_DDL_TX IS NULL
        OR :old.LAST_DDL_TX!=:new.LAST_DDL_TX THEN
            v_other_rec.LAST_DDL_TX:=:new.LAST_DDL_TX;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' LAST_DDL_TX=v_rec.LAST_DDL_TX';
        END IF;                                                                  
       -- LAST_DDL_UDF_DT
        IF :old.LAST_DDL_UDF_DT IS NULL AND :new.LAST_DDL_UDF_DT IS NOT NULL
        OR :old.LAST_DDL_UDF_DT IS NOT NULL and :new.LAST_DDL_UDF_DT IS NULL
        OR :old.LAST_DDL_UDF_DT!=:new.LAST_DDL_UDF_DT THEN
            v_other_rec.LAST_DDL_UDF_DT:=:new.LAST_DDL_UDF_DT;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' LAST_DDL_UDF_DT=v_rec.LAST_DDL_UDF_DT';
        END IF;                      
       -- NAMESPACE
        IF :old.NAMESPACE IS NULL AND :new.NAMESPACE IS NOT NULL
        OR :old.NAMESPACE IS NOT NULL and :new.NAMESPACE IS NULL
        OR :old.NAMESPACE!=:new.NAMESPACE THEN
            v_other_rec.NAMESPACE:=:new.NAMESPACE;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' NAMESPACE=v_rec.NAMESPACE';
        END IF;                                                                                        
       -- OBJECT_ID
        IF :old.OBJECT_ID IS NULL AND :new.OBJECT_ID IS NOT NULL
        OR :old.OBJECT_ID IS NOT NULL and :new.OBJECT_ID IS NULL
        OR :old.OBJECT_ID!=:new.OBJECT_ID THEN
            v_other_rec.OBJECT_ID:=:new.OBJECT_ID;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' OBJECT_ID=v_rec.OBJECT_ID';
        END IF;                                                                                        
       -- OBJECT_NAME
        IF :old.OBJECT_NAME IS NULL AND :new.OBJECT_NAME IS NOT NULL
        OR :old.OBJECT_NAME IS NOT NULL and :new.OBJECT_NAME IS NULL
        OR :old.OBJECT_NAME!=:new.OBJECT_NAME THEN
            v_other_rec.OBJECT_NAME:=:new.OBJECT_NAME;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' OBJECT_NAME=v_rec.OBJECT_NAME';
        END IF;                                                                  
       -- OBJECT_TYPE
        IF :old.OBJECT_TYPE IS NULL AND :new.OBJECT_TYPE IS NOT NULL
        OR :old.OBJECT_TYPE IS NOT NULL and :new.OBJECT_TYPE IS NULL
        OR :old.OBJECT_TYPE!=:new.OBJECT_TYPE THEN
            v_other_rec.OBJECT_TYPE:=:new.OBJECT_TYPE;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' OBJECT_TYPE=v_rec.OBJECT_TYPE';
        END IF;                                                                  
       -- ORACLE_MAINTAINED
        IF :old.ORACLE_MAINTAINED IS NULL AND :new.ORACLE_MAINTAINED IS NOT NULL
        OR :old.ORACLE_MAINTAINED IS NOT NULL and :new.ORACLE_MAINTAINED IS NULL
        OR :old.ORACLE_MAINTAINED!=:new.ORACLE_MAINTAINED THEN
            v_other_rec.ORACLE_MAINTAINED:=:new.ORACLE_MAINTAINED;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' ORACLE_MAINTAINED=v_rec.ORACLE_MAINTAINED';
        END IF;
       -- OWNER
        IF :old.OWNER IS NULL AND :new.OWNER IS NOT NULL
        OR :old.OWNER IS NOT NULL and :new.OWNER IS NULL
        OR :old.OWNER!=:new.OWNER THEN
            v_other_rec.OWNER:=:new.OWNER;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' OWNER=v_rec.OWNER';
        END IF;                                                                                                                                    
       -- SECONDARY
        IF :old.SECONDARY IS NULL AND :new.SECONDARY IS NOT NULL
        OR :old.SECONDARY IS NOT NULL and :new.SECONDARY IS NULL
        OR :old.SECONDARY!=:new.SECONDARY THEN
            v_other_rec.SECONDARY:=:new.SECONDARY;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' SECONDARY=v_rec.SECONDARY';
        END IF;                                                                                        
       -- SHARING
        IF :old.SHARING IS NULL AND :new.SHARING IS NOT NULL
        OR :old.SHARING IS NOT NULL and :new.SHARING IS NULL
        OR :old.SHARING!=:new.SHARING THEN
            v_other_rec.SHARING:=:new.SHARING;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' SHARING=v_rec.SHARING';
        END IF;                                                                                                              
       -- STATUS
        IF :old.STATUS IS NULL AND :new.STATUS IS NOT NULL
        OR :old.STATUS IS NOT NULL and :new.STATUS IS NULL
        OR :old.STATUS!=:new.STATUS THEN
            v_other_rec.STATUS:=:new.STATUS;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' STATUS=v_rec.STATUS';
        END IF;                                                                                                                         
       -- SUBOBJECT_NAME
        IF :old.SUBOBJECT_NAME IS NULL AND :new.SUBOBJECT_NAME IS NOT NULL
        OR :old.SUBOBJECT_NAME IS NOT NULL and :new.SUBOBJECT_NAME IS NULL
        OR :old.SUBOBJECT_NAME!=:new.SUBOBJECT_NAME THEN
            v_other_rec.SUBOBJECT_NAME:=:new.SUBOBJECT_NAME;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' SUBOBJECT_NAME=v_rec.SUBOBJECT_NAME';
        END IF;                                 
       -- TEMPORARY
        IF :old.TEMPORARY IS NULL AND :new.TEMPORARY IS NOT NULL
        OR :old.TEMPORARY IS NOT NULL and :new.TEMPORARY IS NULL
        OR :old.TEMPORARY!=:new.TEMPORARY THEN
            v_other_rec.TEMPORARY:=:new.TEMPORARY;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' TEMPORARY=v_rec.TEMPORARY';
        END IF;                                                                                        
       -- TIMESTAMP
        IF :old.TIMESTAMP IS NULL AND :new.TIMESTAMP IS NOT NULL
        OR :old.TIMESTAMP IS NOT NULL and :new.TIMESTAMP IS NULL
        OR :old.TIMESTAMP!=:new.TIMESTAMP THEN
            v_other_rec.TIMESTAMP:=:new.TIMESTAMP;
            v_otherUpdate_tx:=v_otherUpdate_tx||
                                CASE WHEN v_otherUpdate_tx IS NOT NULL THEN ',' ELSE NULL END
                                ||chr(10)||' TIMESTAMP=v_rec.TIMESTAMP';
        END IF;                                                                                        
    
        v_other_rec.object_id:=:old.object_id;
        v_otherUpdate_tx:=
            'DECLARE '||CHR(10)||
            ' v_rec test_tab_other%ROWTYPE:=:1;'||CHR(10)||
            'BEGIN '||CHR(10)||
            ' UPDATE test_tab_other SET '||
            v_otherUpdate_tx||CHR(10)||
            ' WHERE object_id=v_rec.object_id;'||CHR(10)||
            'END;';
        execute immediate v_otherUpdate_tx using v_other_rec;
    end if;
end;
/
-- Validate performance impact
exec runstats_pkg.rs_start;
ALTER TRIGGER v_test_tab_dynamic_IU DISABLE;
ALTER TRIGGER v_test_tab_IU ENABLE;
begin -- update all columns
    for i in 1..10000 loop
        UPDATE v_test_tab
        SET object_name='Test'||i
        WHERE object_id = 5;
    END LOOP;
END;
/

exec runstats_pkg.rs_middle;
alter trigger v_test_tab_dynamic_IU enable;
alter trigger v_test_tab_IU disable;
BEGIN -- update via Dynamic SQL
    FOR i IN 1..10000 LOOP
        UPDATE v_test_tab
        SET object_name='Test'||i
        WHERE object_id = 5;
    END LOOP;
END;
/
exec runstats_pkg.rs_stop;