-- create table
drop table secure_tab purge
/
CREATE TABLE secure_tab (demo_cl CLOB)
LOB(demo_cl) STORE AS SecureFile demo_seg(COMPRESS HIGH DEDUPLICATE)
/
-- load 20 exactly the same CLOB (each has 1MB of data)
DECLARE
    v_cl CLOB;
BEGIN
    DBMS_LOB.CREATETEMPORARY(v_cl,true,dbms_lob.call);
    FOR i IN 1..250 LOOP
        DBMS_LOB.WRITEAPPEND
            (v_cl,4000,DBMS_RANDOM.STRING('x',4000));
    END LOOP;
    FOR i IN 1..20 LOOP
        INSERT INTO secure_tab VALUES (v_cl);
    END LOOP;
END;
/
DECLARE
    v_tx VARCHAR2(99):='ALTER TABLE secure_tab MODIFY LOB(demo_cl) ';
    PROCEDURE p_print (pi_type_tx VARCHAR2) IS
        v_seg_blocks_nr NUMBER;
        v_seg_bytes_nr NUMBER;
        v_used_blocks_nr NUMBER;
        v_used_bytes_nr NUMBER;
        v_expired_blocks_nr NUMBER;
        v_expired_bytes_nr NUMBER;
        v_unexpired_blocks_nr NUMBER;
        v_unexpired_bytes_nr NUMBER;
    BEGIN
        DBMS_SPACE.SPACE_USAGE (user,'DEMO_SEG','LOB',
            partition_name => NULL,
            segment_size_blocks=> v_seg_blocks_nr,
            segment_size_bytes => v_seg_bytes_nr,
            used_blocks => v_used_blocks_nr,
            used_bytes => v_used_bytes_nr,
            expired_blocks => v_expired_blocks_nr,
            expired_bytes => v_expired_bytes_nr,
            unexpired_blocks => v_unexpired_blocks_nr,
            unexpired_bytes => v_unexpired_bytes_nr);
        
        DBMS_OUTPUT.PUT_LINE 
            (pi_type_tx||':seg-'||v_seg_bytes_nr||
                '/used-'|| v_used_bytes_nr);
    END;
BEGIN
    p_print('High Compress');
    EXECUTE IMMEDIATE v_tx||'(COMPRESS MEDIUM)';
    p_print('Medium Compress');
    EXECUTE IMMEDIATE v_tx||'(COMPRESS LOW)';
    p_print('Low Compress');
    EXECUTE IMMEDIATE v_tx||'(NOCOMPRESS)';
    p_print('No Compress');
    EXECUTE IMMEDIATE v_tx||'(KEEP_DUPLICATES)';
    p_print('Keep dups');
END;
/
--------------------------------------
-- Performance Test
CREATE TABLE secure_compress_tab 
AS 
SELECT * FROM secure_tab
/
ALTER TABLE secure_compress_tab 
    MODIFY LOB (demo_cl) (COMPRESS HIGH)
/
DECLARE
    v_nr number;
BEGIN
    runstats_pkg.rs_start;
    FOR c IN (SELECT * FROM secure_tab) LOOP
        v_nr:=instr(c.demo_cl,'a');
    END LOOP;
    
    runstats_pkg.rs_middle;
    FOR c IN (SELECT * FROM secure_compress_tab) LOOP
         v_nr:=instr(c.demo_cl,'a');
    END LOOP;
    
    runstats_pkg.rs_stop;
END;
/