-- create test table
CREATE TABLE xmlredef_tab
    (id_nr NUMBER PRIMARY KEY,
    data_xml XMLType)
XMLTYPE COLUMN data_xml
STORE AS SecureFile CLOB xmlredef_data_seg (CACHE)
/
-- data loader: 100 rows of 1.5 MB XML data in each
DECLARE
    v_CL CLOB;
    v_tx VARCHAR2(256);
    v_sample_xml XMLType;
BEGIN
    dbms_lob.createTemporary(v_cl,true,dbms_lob.call);
    dbms_lob.writeAppend(v_cl, length('<root>'), '<root>');
    FOR i IN 1..10000 LOOP
        v_tx:='<tag'||i||'>0123456789</tag'||i||'>';
        dbms_lob.writeAppend(v_cl, length(v_tx), v_tx);
    END LOOP;
    dbms_lob.writeAppend(v_cl, length('</root>'), '</root>');
    v_sample_xml:=XMLType(v_cl);

    FOR i IN 1..100 LOOP
        INSERT INTO xmlredef_tab (id_nr,data_xml) VALUES (i,v_sample_xml);
    END LOOP;
END;
/
-- create temporary table
CREATE TABLE xmlredef_tab_tmp
     (id_nr NUMBER PRIMARY KEY,
     data_xml XMLType)
XMLTYPE COLUMN data_xml
STORE AS SecureFile BINARY XML xmlredef_bin_data_seg (CACHE)
/
-- run redefinition
BEGIN
    dbms_redefinition.start_redef_table
        (user, 'XMLREDEF_TAB', 'XMLREDEF_TAB_TMP');
    dbms_redefinition.sync_interim_table
        (user, 'XMLREDEF_TAB', 'XMLREDEF_TAB_TMP');
    dbms_redefinition.finish_redef_table
        (user, 'XMLREDEF_TAB', 'XMLREDEF_TAB_TMP');
END;
/
-- Confirm changes
SELECT table_name, column_name, storage_type
FROM user_xml_tab_cols
WHERE table_name IN ('XMLREDEF_TAB','XMLREDEF_TAB_TMP');

SELECT table_name, segment_name FROM user_lobs
WHERE table_name IN ('XMLREDEF_TAB','XMLREDEF_TAB_TMP');

SELECT segment_name, bytes
FROM user_segments
WHERE segment_name IN ('XMLREDEF_DATA_SEG','XMLREDEF_BIN_DATA_SEG');

-- Performance test
exec runstats_pkg.rs_start;
DECLARE
    v_data_xml XMLType;
    v_value_tx VARCHAR2(1000);
BEGIN
    SELECT data_xml INTO v_data_xml FROM XMLREDEF_TAB_TMP t WHERE id_nr = 1;
    FOR i IN 1000..1100 LOOP
        EXECUTE IMMEDIATE 'SELECT tag_tx
        FROM XMLTable(''/root'' PASSING :1
        COLUMNS tag_tx VARCHAR2(10) PATH ''tag'||i||'''
         )'
        INTO v_value_tx USING v_data_xml;
    END LOOP;
END;
/
exec runstats_pkg.rs_middle;

DECLARE
    v_data_xml XMLType;
    v_value_tx VARCHAR2(1000);
BEGIN
    SELECT data_xml INTO v_data_xml FROM XMLREDEF_TAB t WHERE id_nr = 1;
    FOR i IN 1000..1100 LOOP
        EXECUTE IMMEDIATE 'SELECT tag_tx
        FROM XMLTable(''/root'' PASSING :1
        COLUMNS tag_tx VARCHAR2(10) PATH ''tag'||i||'''
        )'
        INTO v_value_tx USING v_data_xml;
    END LOOP;
END;
/
exec runstats_pkg.rs_stop;