-- create required procedures
CREATE OR REPLACE PROCEDURE p_build_doc IS
    v_CL CLOB;
    v_tx VARCHAR2(256);
    v_parser dbms_xmlparser.Parser;
    v_doc_xml dbms_xmldom.DOMDocument;
BEGIN
    -- initialize LOB and add XML tags into it
    dbms_lob.createTemporary(v_cl,true,dbms_lob.call);
    dbms_lob.writeAppend(v_cl, length('<root>'), '<root>');
    FOR i IN 1..10000 LOOP
        v_tx:='<tag'||i||'>0123456789</tag'||i||'>';
        dbms_lob.writeAppend(v_cl, length(v_tx), v_tx);
    END LOOP;
    
    dbms_lob.writeAppend(v_cl, length('</root>'), '</root>');
    -- create DOM document out of CLOB
    v_parser:=dbms_xmlparser.newParser;
    dbms_xmlparser.parseClob(v_parser, v_CL);
    
    v_doc_xml := dbms_xmlparser.getDocument(v_parser);
    -- process document
    null;
    
    -- release the parser
    dbms_xmlparser.freeParser(v_parser);
END;
/
CREATE OR REPLACE PROCEDURE p_build_doc_free IS
    v_CL CLOB;
    v_tx VARCHAR2(256);
    v_parser dbms_xmlparser.Parser;
    v_doc_xml dbms_xmldom.DOMDocument;
BEGIN
    -- initialize LOB and add XML tags into it
    dbms_lob.createTemporary(v_cl,true,dbms_lob.call);
    dbms_lob.writeAppend(v_cl, length('<root>'), '<root>');
    FOR i IN 1..10000 LOOP
        v_tx:='<tag'||i||'>0123456789</tag'||i||'>';
        dbms_lob.writeAppend(v_cl, length(v_tx), v_tx);
    END LOOP;
    
    dbms_lob.writeAppend(v_cl, length('</root>'), '</root>');
    -- create DOM document out of CLOB
    v_parser:=dbms_xmlparser.newParser;
    dbms_xmlparser.parseClob(v_parser, v_CL);
    
    v_doc_xml := dbms_xmlparser.getDocument(v_parser);
    -- process document
    null;
    
    -- release the parser
    dbms_xmldom.freeDocument(v_doc_xml); -- this line was missing!    
    dbms_xmlparser.freeParser(v_parser);
END;
/
-- Orignal call
BEGIN
    FOR i IN 1..5 LOOP
        p_build_doc;
    END LOOP;
END;
/
-- check the impact
SELECT sid, cache_lobs FROM v$temporary_lobs
WHERE sid = SYS_CONTEXT('userenv','sid');

SELECT category, allocated, used, max_allocated 
FROM v$process_memory
WHERE pid = (SELECT pid 
            FROM v$process 
            WHERE addr =
                (SELECT paddr 
                FROM V$SESSION
                WHERE sid = SYS_CONTEXT('userenv','sid')
                )
            );

SELECT segtype, segfile#, segblk#,extents,blocks 
FROM v$tempseg_usage
WHERE session_addr = (SELECT saddr FROM v$session
                      WHERE sid = SYS_CONTEXT('userenv','sid')
                     );

-- Correct call
connect scott/TIGER@ora11g
BEGIN
    FOR i IN 1..5 LOOP
        p_build_doc_free;
    END LOOP;
END;
/
-- check the impact
SELECT sid, cache_lobs FROM v$temporary_lobs
WHERE sid = SYS_CONTEXT('userenv','sid');

SELECT category, allocated, used, max_allocated 
FROM v$process_memory
WHERE pid = (SELECT pid 
            FROM v$process 
            WHERE addr =
                (SELECT paddr 
                FROM V$SESSION
                WHERE sid = SYS_CONTEXT('userenv','sid')
                )
            );

SELECT segtype, segfile#, segblk#,extents,blocks 
FROM v$tempseg_usage
WHERE session_addr = (SELECT saddr FROM v$session
                      WHERE sid = SYS_CONTEXT('userenv','sid')
                     );