-- object type and collection
CREATE TYPE lov_oty IS OBJECT 
    (id_nr NUMBER,
     display_tx VARCHAR2(4000))
/
CREATE TYPE lov_nt IS TABLE OF lov_oty
/

-- generic value list builder
CREATE FUNCTION f_getlov_nt 
    (i_table_tx VARCHAR2, i_id_tx VARCHAR2,
    i_display_tx VARCHAR2, i_order_tx VARCHAR2)
RETURN lov_nt IS
    v_out_nt lov_nt := lov_nt();
BEGIN
    EXECUTE IMMEDIATE 'SELECT lov_oty('||i_id_tx||','||i_display_tx||
                        ') FROM '||i_table_tx||' ORDER BY '||i_order_tx
    BULK COLLECT INTO v_out_nt;
    RETURN v_out_nt;
END;
/

-- query preparator
CREATE OR REPLACE PROCEDURE p_prepareSQL 
    (i_sql_tx IN VARCHAR2, 
     i_lov_nt IN lov_nt,
     o_cur OUT SYS_REFCURSOR, 
     o_structure_tx OUT VARCHAR2)
IS
    v_cur INTEGER;
    v_result_nr INTEGER;
    v_cols_nr NUMBER := 0;
    v_cols_tt dbms_sql.desc_tab;
BEGIN
    v_cur:=dbms_sql.open_cursor;
    dbms_sql.parse(v_cur, i_sql_tx, dbms_sql.native);
    dbms_sql.describe_columns(v_cur, v_cols_nr, v_cols_tt);
    
    FOR i IN 1 .. v_cols_nr LOOP
        o_structure_tx:=o_structure_tx||'|'||v_cols_tt (i).col_name;
    END LOOP;
    
    dbms_sql.bind_variable(v_cur, 'NT1',i_lov_nt);
    v_result_nr:=dbms_sql.execute(v_cur);
    o_cur:=dbms_sql.to_refcursor(v_cur);
END;    
/

-- put everything together
DECLARE
    v_ref_cur SYS_REFCURSOR;
    v_columnList_tx VARCHAR2(32767);
    v_lov_nt lov_nt:=
        f_getlov_nt('DEPT','DEPTNO','DNAME','DEPTNO');
    v_sql_tx VARCHAR2(32767):=
        'SELECT * '||chr(10)||
        'FROM emp '||chr(10)||
        'WHERE deptno IN ('||chr(10)||
         ' SELECT id_nr'||chr(10)||
         ' FROM TABLE(cast (:NT1 AS lov_nt))'||chr(10)||
         ' )';
BEGIN
    p_prepareSQL(v_sql_tx,
                v_lov_nt,
                v_ref_cur,
                v_columnList_tx);
    dbms_output.put_line('Columns:'||v_columnList_tx);
    IF v_ref_cur%ISOPEN THEN
        dbms_output.put_line('Valid Cursor!');
    END IF;
END;
/
