-- direct approach (dangerous)
CREATE OR REPLACE PROCEDURE p_processObjects (i_list_tx VARCHAR2)
is
    v_sql_tx VARCHAR2(32767);
    v_out_dt DATE;
BEGIN
    v_sql_tx:=
        'SELECT MIN(created) FROM inlist_tab WHERE object_id IN ('||i_list_tx||')';
    EXECUTE IMMEDIATE v_sql_tx INTO v_out_dt;
        dbms_output.put_line('MinDate:'||to_char(v_out_dt,'YYYYMMDD'));
END;
/
exec p_processObjects ('106,25,543');

-- object type approach (requres extra steps)
CREATE TYPE id_tt IS TABLE OF NUMBER
/
CREATE OR REPLACE PROCEDURE p_processObjects (i_tt id_tt)
IS
    v_out_dt DATE;
BEGIN
    SELECT min(created)
    INTO v_out_dt
    FROM inlist_tab
    WHERE object_id IN
        (SELECT /*+ dynamic_sampling(t 2) */ column_value 
         FROM TABLE(i_tt) t);
    dbms_output.put_line('MinDate:'||to_char(v_out_dt,'YYYYMMDD'));
END;
/
exec p_processObjects (id_tt(106,25,543));

-- XML option (balanced)
CREATE OR REPLACE PROCEDURE p_processObjects (i_list_tx VARCHAR2)
IS
    v_out_dt DATE;
BEGIN
    SELECT min(created)
    INTO v_out_dt
    FROM inlist_tab
    WHERE object_id IN
        (SELECT /*+ dynamic_sampling(t 2) */ id_nr
        FROM XMLTABLE(i_list_tx
                        COLUMNS id_nr NUMBER PATH '.') t);
    dbms_output.put_line('MinDate:'||to_char(v_out_dt,'YYYYMMDD'));
 END;
 /
exec p_processObjects ('106,25,543');

-- Enforcing cardinalty
CREATE OR REPLACE PROCEDURE p_processObjects (i_tt id_tt) IS
    v_sql_tx VARCHAR2(32767);
    v_out_dt DATE;
BEGIN
    v_sql_tx:='SELECT min(created) from inlist_tab'||
            ' WHERE object_id IN '||
            '(SELECT /*+ cardinality(t '||i_tt.count||')*/ column_value '||
            ' FROM table(:1) t)';
    EXECUTE IMMEDIATE v_sql_tx INTO v_out_dt USING i_tt;
     dbms_output.put_line('MinDate:'||to_char(v_out_dt,'YYYYMMDD'));
END;
/
exec p_processObjects (id_tt(106,25,543));