-- Create wrapped package for research purposes
CREATE OR REPLACE PACKAGE wrapped_pkg
IS
    FUNCTION f_getData_cl (i_column_tx VARCHAR2, i_table_tx VARCHAR2) RETURN CLOB;
END;
/
declare
    v_cl CLOB;
begin
    v_cl:=dbms_ddl.wrap(    
'CREATE OR REPLACE PACKAGE BODY wrapped_pkg IS
    FUNCTION f_getData_cl(i_column_tx VARCHAR2, i_table_tx VARCHAR2)
    RETURN CLOB
    IS
        v_cl CLOB;
        v_tx VARCHAR2(32767);
        v_cur SYS_REFCURSOR;
    BEGIN
        dbms_lob.createTemporary(v_cl,false,dbms_lob.call);
        OPEN v_cur FOR
            ''SELECT ''||
                dbms_assert.simple_sql_name(i_column_tx)||'' field_tx''||
            '' FROM ''||dbms_assert.simple_sql_name(i_table_tx);
        LOOP
            FETCH v_cur into v_tx;
            EXIT WHEN v_cur%notfound;
            dbms_lob.writeAppend(v_cl, length(v_tx)+1,v_tx||''|'');
        END LOOP;
    
        CLOSE v_cur;
        RETURN v_cl;
    END;
END;');
    
    execute immediate v_cl;    
end;
/

-- Run performance analysis
exec runstats_pkg.rs_start;
DECLARE
    v_CL CLOB;
BEGIN
    v_cl :=wrapped_pkg.f_getdata_cl('ename','emp');
    dbms_output.put_line('length:'||LENGTH(v_cl));
END;
/
exec runstats_pkg.rs_middle;
DECLARE
    v_CL CLOB;
BEGIN
    v_cl :=wrapped_pkg.f_getdata_cl('object_name','test_tab');
    dbms_output.put_line('length:'||LENGTH(v_cl));
END;
/
exec runstats_pkg.rs_stop;

-- Run PL/SQL profiler (assuming destination directory=IO)
DECLARE
    v_CL CLOB;
BEGIN
    dbms_hprof.start_profiling(LOCATION=>'IO', FILENAME=>'Wrap.txt');
    v_cl:=wrapped_pkg.f_getdata_cl('object_name','test_tab');
    dbms_output.put_line('LENGTH:'||length(v_cl));
    dbms_hprof.stop_profiling;
END;
/
