-- Create in-memory storage structure

CREATE TYPE dept_gtt_ot IS OBJECT 
    (deptno NUMBER,
    dname VARCHAR2(14),
    loc VARCHAR2(13))
/    
CREATE TYPE dept_gtt_nt IS TABLE OF dept_gtt_ot
/
CREATE OR REPLACE PACKAGE dept_gtt_pkg IS
    TYPE pk_tt IS TABLE OF NUMBER INDEX BY BINARY_INTEGER;
    v_pk_tt pk_tt;
    v_dept_tt dept_gtt_nt:=dept_gtt_nt();
    FUNCTION f_getDept_tt RETURN dept_gtt_nt;
END;
/
CREATE OR REPLACE PACKAGE BODY dept_gtt_pkg IS
    FUNCTION f_getDept_tt RETURN dept_gtt_nt IS
    BEGIN
        RETURN v_dept_tt;
    END;
END;
/

-- Create view that would simulate a table
CREATE OR REPLACE VIEW v_dept_gtt AS
SELECT *
FROM TABLE(dept_gtt_pkg.f_getDept_tt)
/
CREATE OR REPLACE TRIGGER V_DEPT_GTT_IIUD
INSTEAD OF INSERT OR DELETE OR UPDATE ON V_DEPT_GTT
BEGIN
    IF INSERTING THEN
        IF :new.deptno IS NULL THEN
            raise_application_error(-20001,'Deptno is mandatory');
        END IF;
        IF dept_gtt_pkg.v_pk_tt.exists(:new.deptno) THEN
        raise_application_error(-20001,'Department already exists');
        END IF;
        dept_gtt_pkg.v_dept_tt.extend;
        dept_gtt_pkg.v_pk_tt(:new.deptno):=dept_gtt_pkg.v_dept_tt.last;
        dept_gtt_pkg.v_dept_tt(dept_gtt_pkg.v_dept_tt.last):=
        dept_gtt_ot(:new.deptno,:new.dname,:new.loc);
    ELSIF DELETING THEN
        IF :old.deptno IS NOT NULL AND dept_gtt_pkg.v_pk_tt.exists(:old.deptno) THEN
            dept_gtt_pkg.v_dept_tt.delete(dept_gtt_pkg.v_pk_tt(:old.deptno));
            dept_gtt_pkg.v_pk_tt.delete(:old.deptno);
        END IF;
    ELSIF UPDATING THEN
        IF :new.deptno IS NOT NULL AND :old.deptno!=:new.deptno THEN
            IF dept_gtt_pkg.v_pk_tt.exists(:new.deptno) THEN
                raise_application_error(-20001,'Cannot have duplicate departments');
            ELSE
                dept_gtt_pkg.v_pk_tt(:new.deptno):=dept_gtt_pkg.v_pk_tt(:old.deptno);
                dept_gtt_pkg.v_pk_tt.delete(:old.deptno);
            END IF;
        END IF;
        dept_gtt_pkg.v_dept_tt(dept_gtt_pkg.v_pk_tt(:new.deptno)):=
        dept_gtt_ot(:new.deptno,:new.dname,:new.loc);
    END IF;
END;
/

-- Confirm simulation functionality
INSERT INTO v_dept_gtt SELECT * FROM dept WHERE deptno in (10,20)
/
INSERT INTO v_dept_gtt(deptno,loc) VALUES (50,'NEW JERSEY')
/
UPDATE v_dept_gtt SET dname='DEV' WHERE deptno=50
/
DELETE FROM v_dept_gtt WHERE deptno=20
/
SELECT * FROM v_dept_gtt
/
