-- audit table and trigger
CREATE TABLE ddl_audit_tab (
    ddl_type_tx VARCHAR2(30),
    object_type_tx VARCHAR2(30),
    object_name_tx VARCHAR2(30),
    ddl_date_dt TIMESTAMP,
    code_cl CLOB);

CREATE OR REPLACE TRIGGER ddl_audit_trg 
BEFORE DDL ON SCHEMA
DECLARE
    v_lines_nr PLS_INTEGER;
    v_sql_tt ora_name_list_t;
    v_cl CLOB;
    v_buffer_tx VARCHAR2(32767);
    
    PROCEDURE p_flush IS
    BEGIN
        dbms_lob.writeappend(v_cl,length(v_buffer_tx), v_buffer_tx);
        v_buffer_tx:=NULL;
    END;
    
    PROCEDURE p_add (i_tx VARCHAR2) IS
    BEGIN
        IF length(i_tx)+length(v_buffer_tx)>32767 THEN
            p_flush;
            v_buffer_tx:=i_tx;
        ELSE
            v_buffer_tx:=v_buffer_tx||i_tx;
        END IF;
    END;
BEGIN
    -- security section
    IF ora_dict_obj_name = 'DDL_AUDIT_TAB' THEN
        raise_Application_error(-20001,'Cannot touch DDL_AUDIT_TAB!');
    END IF;
    -- put DDL together
    v_lines_nr := ora_sql_txt(v_sql_tt);
    dbms_lob.createTemporary(v_cl,true,dbms_lob.call);
    FOR i IN 1..v_lines_nr LOOP
        p_add(v_sql_tt(i));
    END LOOP;
    p_flush;
    -- store
    INSERT INTO ddl_audit_tab
        (ddl_type_tx,object_type_tx,object_name_tx,ddl_date_dt,code_cl)
    VALUES
        (ora_sysevent,ora_dict_obj_type,ora_dict_obj_name,SYSTIMESTAMP,v_cl);
END;
/
-- Validate functionality
CREATE TABLE tst1(a number)
/
SELECT * FROM ddl_audit_tab
/
TRUNCATE TABLE ddl_audit_tab
/
