-- Deployed module
CREATE FUNCTION f_getEmp_CL (i_job_tx VARCHAR2, i_hiredate_dt DATE:=NULL)
RETURN CLOB
IS
    v_out_cl CLOB;

    PROCEDURE p_add(pi_tx VARCHAR2) IS BEGIN
        dbms_lob.writeappend(v_out_cl,length(pi_tx),pi_tx);
    END;
BEGIN
    dbms_lob.createtemporary(v_out_cl,true,dbms_lob.call);
    p_add('<html><table>');
    FOR c IN (SELECT '<tr>'||'<td>'||empno||'</td>'||
                    '<td>'||ename||'</td>'||'</tr>' row_tx
                FROM emp
                WHERE job = i_job_tx
                AND hiredate >= NVL(i_hiredate_dt,add_months(sysdate,-36))
            ) LOOP
        p_add(c.row_tx);
    END LOOP;
    
    p_add('</table></html>');
    RETURN v_out_cl;
END;
/
-- Repository
CREATE TABLE module_tab
    (module_id NUMBER PRIMARY KEY,
    displayName_tx VARCHAR2(256),
    module_tx VARCHAR2(50),
    v1_label_tx VARCHAR2(100),
    v1_type_tx VARCHAR2(50),
    v1_required_yn VARCHAR2(1),
    v1_lov_tx VARCHAR2(50),
    v1_convert_tx VARCHAR2(50),
    v2_label_tx VARCHAR2(100),
    v2_type_tx VARCHAR2(50),
    v2_required_yn VARCHAR2(1),
    v2_lov_tx VARCHAR2(50),
    v2_convert_tx VARCHAR2(50))
/
CREATE TABLE edition_tab
    (edition_id NUMBER PRIMARY KEY,
    name_tx VARCHAR2(50),
    edition_rfk NUMBER) -- Recursive FK
/    
CREATE TABLE module_edition
    (module_edition_id NUMBER PRIMARY KEY,
    module_id NUMBER, -- FK
    edition_id NUMBER) -- FK
/
CREATE TABLE user_edition
    (user_edition_id NUMBER PRIMARY key,
    user_tx VARCHAR2(30),
    edition_id NUMBER) --FK
/

-- Repository data
INSERT INTO module_tab (module_id,displayName_tx,module_tx,
	v1_label_tx, v1_type_tx, v1_required_yn,
	v2_label_tx, v2_type_tx, v2_required_yn, v2_convert_tx)
VALUES (100, 'Filter Employees by Job/Hire Date', 'f_getEmp_cl',
	'Job','TEXT','Y',
	'Hire Date','DATE','N','TO_DATE(v2_tx,''YYYYMMDD'')')
/
INSERT INTO module_tab (module_id,displayName_tx,module_tx,
	v1_label_tx, v1_type_tx, v1_required_yn)
VALUES (101, 'Filter Employees by Job', 'f_getEmp_cl',
	'Job','TEXT','Y')
/
INSERT INTO edition_tab (edition_id, name_tx, edition_rfk) VALUES (10, 'Default Edition',NULL)
/
INSERT INTO edition_tab (edition_id, name_tx, edition_rfk) VALUES (11, 'New Edition',10)
/
INSERT INTO module_edition (module_edition_id,module_id,edition_id) values (20,100,10)
/
INSERT INTO module_edition (module_edition_id,module_id,edition_id) values (21,101,11)
/
INSERT INTO user_edition (user_edition_id, user_tx, edition_id) values (30,'HR',10)
/
INSERT INTO user_edition (user_edition_id, user_tx, edition_id) values (31,'OE',11)
/

-- Retrieve repository data
SELECT m.module_id, m.displayname_tx
FROM module_tab m,
    module_edition me
WHERE m.module_id = me.module_id
AND me.edition_id IN (SELECT edition_id
                        FROM user_edition
                        WHERE user_tx = 'HR')
/
SELECT m.module_id, m.displayname_tx
FROM module_tab m,
    module_edition me
WHERE m.module_id = me.module_id
AND me.edition_id in (SELECT edition_id
                    FROM user_edition
                    WHERE user_tx = 'OE')
/                    

-- Create generic wrapper to call modules
CREATE OR REPLACE FUNCTION f_wrapper_cl 
    (i_module_id NUMBER,
    i_v1_tx VARCHAR2:=null,
    i_v2_tx VARCHAR2:=null)
RETURN CLOB
IS
    v_out_cl CLOB;
    v_sql_tx VARCHAR2(32767);
    v_rec module_tab%ROWTYPE;
BEGIN
    SELECT * INTO v_rec FROM module_tab WHERE module_id=i_module_id;
    
    IF v_rec.v1_label_tx IS NOT NULL THEN
        v_sql_tx:=nvl(v_rec.v1_convert_tx,'v1_tx');
    END IF;
    
    IF v_rec.v2_label_tx IS NOT NULL THEN
        v_sql_tx:=v_sql_tx||','||nvl(v_rec.v2_convert_tx,'v2_tx');
    END IF;
    
    v_sql_tx:=
        'DECLARE '||chr(10)||
        ' v1_tx VARCHAR2(32767):=:1;'||CHR(10)||
        ' v2_tx VARCHAR2(32767):=:2;'||CHR(10)||
        'BEGIN '||CHR(10)||
        ' :out:='||v_rec.module_tx||'('||v_sql_tx||');'||CHR(10)||
        'END;';
    EXECUTE IMMEDIATE v_sql_tx USING i_v1_tx,i_v2_tx, OUT v_out_cl;
    RETURN v_out_cl;
END;
/
-- Validate functionality
SELECT f_wrapper_cl (100,'PRESIDENT','19001010') 
FROM DUAL
/