-- test function
CREATE OR REPLACE FUNCTION f_isEastCoast_yn (i_deptno NUMBER)
RETURN VARCHAR2
IS
    v_out_yn varchar2(1);
BEGIN
    SELECT CASE WHEN loc IN ('NEW YORK', 'BOSTON') THEN 'Y'
                ELSE 'N'
            END
    INTO v_out_yn
    FROM dept
    WHERE deptno = i_deptno;
    RETURN v_out_yn;
END;
/
-- direct use of function
CREATE MATERIALIZED VIEW v_emp_mv
AS
SELECT emp.empno, emp.ename, emp.deptno, f_isEastCoast_yn(deptno) eastCoast_yn
FROM emp
/
-- check datatype - for the function-based column it will be VARCHAR2(4000)
SELECT column_name, data_type, data_length
FROM user_tab_columns
WHERE table_name = 'V_EMP_MV'
ORDER by column_id
/
-- recreate materialized view and explicitly specify the datatype
DROP MATERIALIZED VIEW v_emp_mv
/
CREATE MATERIALIZED VIEW v_emp_mv AS
    SELECT emp.empno, emp.ename, emp.deptno,
            cast(f_isEastCoast_yn(deptno) as VARCHAR2(1)) eastCoast_yn
    fROM emp
/
-- validate the datatype
SELECT column_name, data_type, data_length
FROM user_tab_columns
WHERE table_name = 'V_EMP_MV'
ORDER by column_id
/
-- columns in regular views would also look like VARCHAR2(4000) if you do not specify the datatype
CREATE OR REPLACE VIEW v_emp_app AS
SELECT emp.empno, emp.ename, emp.deptno,
    f_isEastCoast_yn(deptno) eastCoast_yn
FROM emp
/
SELECT column_name, data_type, data_length
FROM user_tab_columns
WHERE table_name = 'V_EMP_APP'
ORDER by column_id
/